#include <sstream>
#include <iostream>
#include <algorithm>
#include "polynomial.h"
using namespace std;

int main(int argc, char** argv)
{
    if(argc > 1)
    {
        // read parameters
        vector<int> branchingvector(argc-1);
        for(int i = 1; i < argc; i++)
        {
            stringstream s;
            s << argv[i];
            s >> branchingvector[i-1];
        }

        if(*min_element(branchingvector.begin(), branchingvector.end()) >= 1)
        {
            // construct characteristic polynomial
            int maxn = *max_element(branchingvector.begin(), branchingvector.end());
            polynomial<long double> characteristicpolynomial((size_t)maxn);
            characteristicpolynomial[maxn] = 1;
            for(vector<int>::iterator i = branchingvector.begin(); i != branchingvector.end(); i++)
                characteristicpolynomial[maxn-*i]--;
            characteristicpolynomial.RemoveLeadingZeros();
            cout << "characteristic polynomial: " << characteristicpolynomial.LaTeX() << "\n";
            
            
            // choose largest zero of characteristic polynomial
            vector<long double> zeroes = characteristicpolynomial.Zeroes<long double>();
            for(vector<long double>::iterator i = zeroes.begin(); i != zeroes.end(); i++)
                if(*i < 0)
                    *i = *i * -1;
            vector<long double>::iterator m = max_element(zeroes.begin(), zeroes.end());
            if(m != zeroes.end())
                cout << "maximal zero: " << *m << "\n";
            else
                cerr << "could not find a zero of the characteristic polynomial :-(\n";
        }
        else
        {
            cerr << "all entries in the branching vector must be >= 1 (0 makes no sense - the recursion tree would be infinite)\n";
            return 1;
        }
    }
    else
    {
        cerr << "usage: " << argv[0] << " {num}+\n";
        return 1;
    }
    
    return 0;
}