
#ifndef __POLYNOMIAL_H
    #define __POLYNOMIAL_H

    
    #include<sstream>
#include<iostream>
    #include<vector>

    
    // polynomial =  coefficients[n]*X^n + ... + coefficients[1]*X + coefficients[0]
    
    template<class T> class polynomial
    {
        template<class X> friend std::ostream& operator << (std::ostream& os, const polynomial<X>& Poly);
        template<class X> friend std::istream& operator >> (std::istream& is, polynomial<X>& Poly);

        protected:
            std::vector<T> coefficients;
            template<class U> int signum(const U& x) const; // why isnt this in stdlib or something?
            template<class U> U ApproximateZero(U lower, U upper) const;
            template<class U> U FindZero(U start, U direction) const;
            
        public:

            polynomial(size_t grade, T *coeffs = NULL);                     // System routines
            polynomial(const std::vector<T> &X);
            polynomial(const polynomial<T>& X);
            polynomial();
            polynomial(T);
            virtual void operator=(const polynomial<T>& a);
            virtual size_t Grade() const;
            virtual const T& operator[](size_t index) const;
            virtual T& operator[](size_t index);
            template<class U> U operator()(const U& x) const;

            virtual bool operator==(const polynomial<T>& a) const;          // comparison
            virtual bool operator!=(const polynomial<T>& a) const;
            virtual bool operator< (const polynomial<T>& a) const;
            virtual bool operator<=(const polynomial<T>& a) const;
            virtual bool operator> (const polynomial<T>& a) const;
            virtual bool operator>=(const polynomial<T>& a) const;

            virtual polynomial<T> operator+(const polynomial<T>& a) const;  // operators
            virtual polynomial<T> operator-(const polynomial<T>& a) const;
            virtual polynomial<T> operator*(const polynomial<T>& a) const;
            virtual polynomial<T> operator/(const polynomial<T>& a) const;
            virtual polynomial<T> operator%(const polynomial<T>& a) const;

            virtual void operator+=(const polynomial<T>& a);
            virtual void operator-=(const polynomial<T>& a);
            virtual void operator*=(const polynomial<T>& a);
            virtual void operator/=(const polynomial<T>& a);
            virtual void operator%=(const polynomial<T>& a);

            virtual void RemoveLeadingZeros();
            virtual polynomial<T> Derivation() const;                       // polynomial-specific methods
            virtual polynomial<T> Integral() const;
            virtual polynomial<T> Normalize() const;
            virtual bool Normal() const;
            template<class U> std::vector<U> Zeroes() const;
            virtual std::string LaTeX(const char* OpeningBracket="", const char* ClosingBracket="") const;
    };

    template<class T> std::ostream& operator << (std::ostream& os, const polynomial<T>& p);
    template<class T> std::istream& operator >> (std::istream& is, polynomial<T>& p);


    #include "polynomial.cpp"
    
#endif
